task-semver(7) -- semver task(s) for bake(1)
=============================================

## SYNOPSIS

	bake semver [command] [options...]

## DESCRIPTION

Task(s) for using the semantic versioning specification in a project.

Command methods for semantic versioning, including semver(7) validation and creation of a project version descriptor.

## REQUIRE

In your tasks(7) file `require` the `semver` task(s) using:

	require 'tasks/semver';

## COMMANDS

If no command is specified then `init` is assumed.

* `init`:
	
Generates the initial task-semver(7) files.

If no files are available you will be prompted to enter a semver(7) compatible version to initialize the files with.
	
If a *package.json* file is available then the semver(7) string is read from the `version` field and the `version` and `semver.json` files are generated.

If all files exist this command is equivalent to `read`.

* `major`:

Bump the major version.

* `minor`:

Bump the minor version.

* `patch`:

Bump the patch version.
	
* `read`:
	
Print version information for the files managed by task-semver(7).
	
If no task-semver(7) files are present this command will exit with a >0 exit code, otherwise it will print warnings for any missing files.

* `write [semver]`:
	
Write *semver* to disc as the version in use. Must be a valid semver(7) string.

* `sync`:

Use the `sync` command to test whether all files are in sync.
	
If any semver(7) strings do not match warnings will be printed and the exit code will be >0.

* `test [semver]`:

Test if a semver(7) string is valid.
	
## PUBLIC RELEASE

A *public release* is defined as going from major version 0 to major version 1. This section describe the process involved with performing a *public release*. Note that the interactivity described here only applies when stdin is a tty.

### Confirm

Initially task-semver(7) will display a verbose warning that you are about to perform a *public release* and a prompt asking you whether you wish to proceed.

If you accept the process and the new version is not a 1.0.0 version (for example you may be bumping to 1.1.0) then a prompt is displayed asking whether you wish to use a 1.0.0 version. Upon accepting this prompt the new version becomes 1.0.0, otherwise it will remain at the version originally specified.

Subsequently, the pre-processing stage begins which attempts to perform various actions that verify the code integrity.

### Test

The first phase is to run unit tests using `npm test` if a *package.json* file is available or alternatively running the task-test(7) test command using bake(1). The stdout from running the tests are redirected to `${target}/release.test.log` whilst stderr will remain a tty if run from a tty (which allows faster debugging if some tests are failing).

If the exit code is >0 when running the tests the release process is aborted.
	
## FILES

All files are stored in the `${root}` of the project. The rules for file precedence are the order described below, so a npm(1) *package.json* takes precedence, followed by a *version* file and finally a *semver.json* file is used if available.

### Package

An npm(1) package descriptor (*package.json*) that stores semver(7) information in the `version` field.

### Version

A text file (*version*) containing the current semver(7) version. This file is generated and managed because there are occassions when a program requires quick access to version information without any parsing. A quick way to access semver(7) information from a strike(7) program is:

	version() {
		local file="${process_dirs[root]}/version";
		local semver=$( cat "$file" );
		console info "version %s" "$semver";
	}
	version;

### Semver

A JSON document (*semver.json*) generated by task-semver(7) containing version information for the project.

### Backup

If a *package.json* file is available and sed(1) is available and used to replace the version field, then a *package.json.bak* file is created and placed in *target*.

## EXAMPLES

Initialize the files:

	bake semver init
	
Bump the patch version:

	bake semver patch
	
Bump the minor version:

	bake semver minor
	
Bump the major version:

	bake semver major
	
Write a specific semver version to all files:

	bake semver write 0.1.0
	
View the semver information:

	bake semver read
	
Attempt to synchronize (when versions are out of sync):

	bake semver sync

Validate a semver string:

	bake semver test 1.2.0-alpha.1+build.12f4e9

## BUGS

**task-semver** is written in bash and depends upon `bash` >= 4.2.

## COPYRIGHT

**task-semver** is copyright (c) 2012 muji <http://xpm.io>

## SEE ALSO

bake(1), semver(3), semver(7)


[SYNOPSIS]: #SYNOPSIS "SYNOPSIS"
[DESCRIPTION]: #DESCRIPTION "DESCRIPTION"
[REQUIRE]: #REQUIRE "REQUIRE"
[COMMANDS]: #COMMANDS "COMMANDS"
[PUBLIC RELEASE]: #PUBLIC-RELEASE "PUBLIC RELEASE"
[Confirm]: #Confirm "Confirm"
[Test]: #Test "Test"
[FILES]: #FILES "FILES"
[Package]: #Package "Package"
[Version]: #Version "Version"
[Semver]: #Semver "Semver"
[Backup]: #Backup "Backup"
[EXAMPLES]: #EXAMPLES "EXAMPLES"
[BUGS]: #BUGS "BUGS"
[COPYRIGHT]: #COPYRIGHT "COPYRIGHT"
[SEE ALSO]: #SEE-ALSO "SEE ALSO"


[strike(1)]: strike.1.html
[boilerplate(3)]: boilerplate.3.html
[require(3)]: require.3.html
[method(3)]: method.3.html
[http(3)]: http.3.html
[bake(1)]: bake.1.html
[rest(1)]: rest.1.html
[bash(1)]: http://man.cx/bash(1)
[curl(1)]: http://man.cx/curl(1)
[echo(1)]: http://man.cx/echo(1)
[find(1)]: http://man.cx/find(1)
[tee(1)]: http://man.cx/tee(1)
[sed(1)]: http://man.cx/sed(1)
[printf(1)]: http://man.cx/printf(1)
[source(1)]: http://man.cx/source(1)
[dirname(1)]: http://man.cx/dirname(1)
[basename(1)]: http://man.cx/basename(1)
[tar(1)]: http://man.cx/tar(1)
[zip(1)]: http://man.cx/zip(1)
[unzip(1)]: http://man.cx/unzip(1)
[compress(1)]: http://man.cx/compress(1)
[gzip(1)]: http://man.cx/gzip(1)
[gunzip(1)]: http://man.cx/gunzip(1)
[pdflatex(1)]: http://man.cx/pdflatex(1)
[openssl(1)]: http://man.cx/openssl(1)
[scp(1)]: http://man.cx/scp(1)
[ssh(1)]: http://man.cx/ssh(1)
[rsync(1)]: http://man.cx/rsync(1)
[autoreconf(1)]: http://man.cx/autoreconf(1)
[checkbashisms(1)]: http://man.cx/checkbashisms
[growlnotify(1)]: http://scottlab.ucsc.edu/Library/init/zsh/man/html/growlnotify.html
[sendmail(1)]: http://man.cx/sendmail(1)
[uuencode(1)]: http://man.cx/uuencode(1)
[epxand(1)]: http://man.cx/expand(1)
[unepxand(1)]: http://man.cx/unexpand(1)
[git(1)]: http://git-scm.com/
[ronn(1)]: https://github.com/rtomayko/ronn
[github(7)]: http://github.com/
[json-sh(1)]: https://github.com/dominictarr/JSON.sh
[npm(1)]: http://npmjs.org
[ruby(3)]: http://www.ruby-lang.org/
[rake(1)]: http://rake.rubyforge.org/
[semver(7)]: http://semver.org/
[ant(1)]: http://ant.apache.org/
[mvn(1)]: http://maven.apache.org/
[make(1)]: http://www.gnu.org/software/make/
[jsonlint(1)]: https://github.com/zaach/jsonlint
[jsoncheck(1)]: http://json.org/JSON_checker/
[ere(7)]: http://pubs.opengroup.org/onlinepubs/9699919799/basedefs/V1_chap09.html
[couchdb(7)]: http://couchdb.apache.org/
[url(7)]: http://www.ietf.org/rfc/rfc1738.txt
[array-file(3)]: array-file.3.html
[array(3)]: array.3.html
[console(1)]: console.1.html
[console(3)]: console.3.html
[delegate(3)]: delegate.3.html
[executable(3)]: executable.3.html
[git(3)]: git.3.html
[globals(3)]: globals.3.html
[help(3)]: help.3.html
[json(3)]: json.3.html
[manual(1)]: manual.1.html
[prompt(1)]: prompt.1.html
[prompt(3)]: prompt.3.html
[semver(3)]: semver.3.html
[sprintf(3)]: sprintf.3.html
[strike-credits(7)]: strike-credits.7.html
[strike-tree(7)]: strike-tree.7.html
[strike(7)]: strike.7.html
[task-ant(7)]: task-ant.7.html
[task-archive(7)]: task-archive.7.html
[task-clean(7)]: task-clean.7.html
[task-compress(7)]: task-compress.7.html
[task-deploy-json(7)]: task-deploy-json.7.html
[task-deploy(7)]: task-deploy.7.html
[task-devel(7)]: task-devel.7.html
[task-doc(7)]: task-doc.7.html
[task-expand(7)]: task-expand.7.html
[task-latex(7)]: task-latex.7.html
[task-ls(7)]: task-ls.7.html
[task-make(7)]: task-make.7.html
[task-module(7)]: task-module.7.html
[task-mvn(7)]: task-mvn.7.html
[task-project(7)]: task-project.7.html
[task-rake(7)]: task-rake.7.html
[task-semver(7)]: task-semver.7.html
[task-test(7)]: task-test.7.html
[task-todo(7)]: task-todo.7.html
[version(3)]: version.3.html
