task-test(7) -- unit test task(s) for bake(1)
=============================================

## SYNOPSIS

Unit test task(s) for bake(1).

## DESCRIPTION

Adds task methods for unit testing bash(1) programs, unit tests are defined in files with a `.test` extension or as standalone executables with the `test/bin` directory.

Binary tests (in `test/bin`) are determined using the `-x` file test flag therefore it is important to ensure you *do not make files executable that are not test executables*.

## REQUIRE

In your tasks(7) file `require` the `test` task(s) using:

	require 'tasks/test';

## USAGE

	bake test [command] [suite:test...]
	
When no test suites are specified all test suites will be executed, otherwise pass a list of specific tests to run. Note that when specifying the test suites you *do not* include the `.test` file extension.

## COMMANDS

* `run`:

	Run one or more test suites.
	
* `ls`:

	List test suites and test methods (if test methods have been specified). Use this command to test which test suites would be matched by the commands passed to task-test(7).
	
	If some tests have been excluded they are listed at the end with each test suite name preceeded by an exclamation mark '!'.
	
* `make`:

	Attempts to run a make(1) *test* target.
	
* `rake`:

	Attempts to run a rake(1) *test* task.
	
* `ant`:

	Attempts to run an ant(1) *test* target.
	
* `mvn`:

	Attempts to run a mvn(1) *test* goal.
	
## GLOBALS

If you want to declare variables or methods global to all test suites place them in `lib/globals` relative to the test directory. They will be sourced prior to running any tests (and prior to running any declared global setup command).
	
## SETUP & TEARDOWN

This section describes how to configure *setup* and *teardown* for all test suites or for individual test suites.

### Global

Global *setup* and *teardown* commands are executed before and after the test runner runs all suites or specific suites.

To configure global *setup* and *teardown* methods create the files `lib/setup` and `lib/teardown` relative to the test directory and within each file declare the `tdd.setup` and `tdd.teardown` commands respectively.

### Suite

To declare *setup* and *teardown* commands specific to a test suite declare the `tdd.setup` and `tdd.teardown` commands within the test suite and they will be invoked before and after all `test.*` commands for that suite.
	
## EXAMPLES

These examples are based on the unit tests in the `test` directory of the strike(7) repository.

The following commands are equivalent for running all tests:

	bake test
	bake test run
	
Run an individual test suite:

	bake test run json.stringify
	
Run multiple test suites:

	bake test run json.stringify json.checker
	
Run a single method of a test suite:

	bake test run json.stringify:basic
	
Note you omit the `test` prefix as well as the test suite prefix `json.stringify` when referencing method names.

Run multiple methods of a test suite delimited by the colon ':' character:

	bake test run json.stringify:basic:boolean
	
It is an error condition to have a leading or trailing colon and the exit code will be >0 in this situation.

## FILES

The `stdout` generated by unit tests is written to `${target}/test.stdout.log` whilst the `stderr` output from the unit tests is written to `${target}/test.stderr.log`.

## BUGS

**task-test** is written in bash and depends upon `bash` >= 4.2.

## COPYRIGHT

**task-test** is copyright (c) 2012 muji <http://xpm.io>

## SEE ALSO

bake(1), assert(7)


[SYNOPSIS]: #SYNOPSIS "SYNOPSIS"
[DESCRIPTION]: #DESCRIPTION "DESCRIPTION"
[REQUIRE]: #REQUIRE "REQUIRE"
[USAGE]: #USAGE "USAGE"
[COMMANDS]: #COMMANDS "COMMANDS"
[GLOBALS]: #GLOBALS "GLOBALS"
[Global]: #Global "Global"
[Suite]: #Suite "Suite"
[EXAMPLES]: #EXAMPLES "EXAMPLES"
[FILES]: #FILES "FILES"
[BUGS]: #BUGS "BUGS"
[COPYRIGHT]: #COPYRIGHT "COPYRIGHT"
[SEE ALSO]: #SEE-ALSO "SEE ALSO"


[strike(1)]: strike.1.html
[boilerplate(3)]: boilerplate.3.html
[require(3)]: require.3.html
[method(3)]: method.3.html
[http(3)]: http.3.html
[bake(1)]: bake.1.html
[rest(1)]: rest.1.html
[bash(1)]: http://man.cx/bash(1)
[curl(1)]: http://man.cx/curl(1)
[echo(1)]: http://man.cx/echo(1)
[find(1)]: http://man.cx/find(1)
[tee(1)]: http://man.cx/tee(1)
[sed(1)]: http://man.cx/sed(1)
[printf(1)]: http://man.cx/printf(1)
[source(1)]: http://man.cx/source(1)
[dirname(1)]: http://man.cx/dirname(1)
[basename(1)]: http://man.cx/basename(1)
[tar(1)]: http://man.cx/tar(1)
[zip(1)]: http://man.cx/zip(1)
[unzip(1)]: http://man.cx/unzip(1)
[compress(1)]: http://man.cx/compress(1)
[gzip(1)]: http://man.cx/gzip(1)
[gunzip(1)]: http://man.cx/gunzip(1)
[pdflatex(1)]: http://man.cx/pdflatex(1)
[openssl(1)]: http://man.cx/openssl(1)
[scp(1)]: http://man.cx/scp(1)
[ssh(1)]: http://man.cx/ssh(1)
[rsync(1)]: http://man.cx/rsync(1)
[autoreconf(1)]: http://man.cx/autoreconf(1)
[checkbashisms(1)]: http://man.cx/checkbashisms
[growlnotify(1)]: http://scottlab.ucsc.edu/Library/init/zsh/man/html/growlnotify.html
[sendmail(1)]: http://man.cx/sendmail(1)
[uuencode(1)]: http://man.cx/uuencode(1)
[epxand(1)]: http://man.cx/expand(1)
[unepxand(1)]: http://man.cx/unexpand(1)
[git(1)]: http://git-scm.com/
[ronn(1)]: https://github.com/rtomayko/ronn
[github(7)]: http://github.com/
[json-sh(1)]: https://github.com/dominictarr/JSON.sh
[npm(1)]: http://npmjs.org
[ruby(3)]: http://www.ruby-lang.org/
[rake(1)]: http://rake.rubyforge.org/
[semver(7)]: http://semver.org/
[ant(1)]: http://ant.apache.org/
[mvn(1)]: http://maven.apache.org/
[make(1)]: http://www.gnu.org/software/make/
[jsonlint(1)]: https://github.com/zaach/jsonlint
[jsoncheck(1)]: http://json.org/JSON_checker/
[ere(7)]: http://pubs.opengroup.org/onlinepubs/9699919799/basedefs/V1_chap09.html
[couchdb(7)]: http://couchdb.apache.org/
[url(7)]: http://www.ietf.org/rfc/rfc1738.txt
[array-file(3)]: array-file.3.html
[array(3)]: array.3.html
[console(1)]: console.1.html
[console(3)]: console.3.html
[delegate(3)]: delegate.3.html
[executable(3)]: executable.3.html
[git(3)]: git.3.html
[globals(3)]: globals.3.html
[help(3)]: help.3.html
[json(3)]: json.3.html
[manual(1)]: manual.1.html
[prompt(1)]: prompt.1.html
[prompt(3)]: prompt.3.html
[semver(3)]: semver.3.html
[sprintf(3)]: sprintf.3.html
[strike-credits(7)]: strike-credits.7.html
[strike-tree(7)]: strike-tree.7.html
[strike(7)]: strike.7.html
[task-ant(7)]: task-ant.7.html
[task-archive(7)]: task-archive.7.html
[task-clean(7)]: task-clean.7.html
[task-compress(7)]: task-compress.7.html
[task-deploy-json(7)]: task-deploy-json.7.html
[task-deploy(7)]: task-deploy.7.html
[task-devel(7)]: task-devel.7.html
[task-doc(7)]: task-doc.7.html
[task-expand(7)]: task-expand.7.html
[task-latex(7)]: task-latex.7.html
[task-ls(7)]: task-ls.7.html
[task-make(7)]: task-make.7.html
[task-module(7)]: task-module.7.html
[task-mvn(7)]: task-mvn.7.html
[task-project(7)]: task-project.7.html
[task-rake(7)]: task-rake.7.html
[task-semver(7)]: task-semver.7.html
[task-test(7)]: task-test.7.html
[task-todo(7)]: task-todo.7.html
[version(3)]: version.3.html
